/*
   Copyright 2013 Unicamp / CCUEC / DDS / CTMQ

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License. 
 */
package br.unicamp.ccuec.ctmq.virusScan.impl;

import java.net.InetAddress;
import java.net.UnknownHostException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.logging.Level;
import java.util.logging.Logger;

import br.unicamp.ccuec.ctmq.util.MailHelper;
import br.unicamp.ccuec.ctmq.virusScan.spec.VirusScan;

/**
 * Thread para em paralelo executar ao longo do dia auto-testes solucao de anti-virus na servidora.
 * Em caso do anti-virus nao responder, uma mensagem de erro sera logada bem como e-mail de alerta sera enviado.
 * 
 * @author clayton@ccuec.unicamp.br (Denis Clayton Alves Ramos)
 * 
 */
public class RunAutoTest implements Runnable {
	private static final String _MSG_SUBJECT = " N\u00c3O EST\u00c1 RESPONDENDO!!! UPLOAD DOCUMENTOS EM RISCO!!!";
	private static final String _MSG_BODY = "CHECAR URGENTE SE ANTI-VIRUS EST\u00c1 EXECUTANDO EM: ";

	private static SimpleDateFormat _sdf = new SimpleDateFormat(
			"dd/MM/yyyy HH:mm");

	private final String[] alertMailsTo;
	private final String[] alertMailsCc;
	private final String msgError;
	private final Logger logger;

	public RunAutoTest(String[] alertMailsTo, String[] alertMailsCc,
			String msgError, Logger logger) {
		this.alertMailsTo = alertMailsTo;
		this.alertMailsCc = alertMailsCc;
		this.msgError = msgError;
		this.logger = logger;
	}

	@Override
	public void run() {
		VirusScan scanner = AbstractVirusScan.getScanner();

		do {
			try {
				if (!scanner.ping()) {
					String hostName = "";
					try {
						hostName = InetAddress.getLocalHost().getHostName();
					} catch (UnknownHostException e) {
						e.printStackTrace();
					}

					MailHelper.alert(alertMailsTo, alertMailsCc, msgError
							+ _MSG_SUBJECT, _MSG_BODY + hostName
							+ ". AVISAR DDS / DSSOF.");

					logger.log(Level.INFO, msgError + _MSG_SUBJECT
							+ ". AVISAR DDS / DSSOF.\n\n");
				}
				else {
					logger.log(Level.INFO,
							"\n\n***** INFRA ANTI-VIRUS RESPONDENDO OK!!! "
									+ _sdf.format(new Date().getTime()));
				}
				//a cada 20'
				Thread.sleep(1200000);
			}
			catch (InterruptedException e) {
				logger.log(Level.INFO, "\n\n***** Thread de espera para checar anti-virus interrompida: "
						+ e.getMessage());
			}
		} while (true);
	}
}