/*
   Copyright 2013 Unicamp / CCUEC / DDS / CTMQ

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License. 
 */
package br.unicamp.ccuec.ctmq.virusScan.impl;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.util.logging.Level;
import java.util.logging.Logger;

import br.unicamp.ccuec.ctmq.virusScan.spec.InfraException;
import br.unicamp.ccuec.ctmq.virusScan.spec.VirusException;
import br.unicamp.ccuec.ctmq.virusScan.spec.VirusScan;

import com.philvarner.clamavj.ClamScan;
import com.philvarner.clamavj.ScanResult;

/**
 * Classe cliente do anti-virus clamavis para checagem de virus.
 * 
 * @author clayton@ccuec.unicamp.br (Denis Clayton Alves Ramos)
 * 
 */
public class VirusScanClamavis extends AbstractVirusScan implements VirusScan {
	private static final Logger _LOGGER = Logger
			.getLogger(VirusScanClamavis.class.getName());

	private ClamScan clamav = null;

	public VirusScanClamavis() {
		clamav = new ClamScan(host, port, timeout);
	}

	@Override
	public boolean ping() {
		return clamav.ping();
	}

	/**
	 * @see br.unicamp.ccuec.ctmq.virusScan.spec.VirusScan#scan(java.lang.String)
	 */
	@Override
	public boolean scan(String path) throws VirusException, InfraException {
		File file = null;
		FileInputStream fis = null;

		try {
			file = new File(path);
			fis = new FileInputStream(file);

			scan(fis);
		} catch (FileNotFoundException e) {
			_LOGGER.log(Level.WARNING, " path " + path
					+ " N\u00c3O PODE SER ESCANEADO!");
			e.printStackTrace();
		} finally {
			try {
				if (fis != null) {
					fis.close();
				}
			} catch (IOException ioe) {
				_LOGGER.log(Level.WARNING, path
						+ " N\u00c3O DESCARREGADO DA MEM\u00d3RIA!");
				ioe.printStackTrace();
			}
		}
		return true;
	}

	/**
	 * @see br.unicamp.ccuec.ctmq.virusScan.spec.VirusScan#scan(byte[])
	 */
	@Override
	public boolean scan(byte[] bytes) throws VirusException, InfraException {
		ScanResult result;
		try {
			result = clamav.scan(bytes);
			if (result.getStatus() == ScanResult.Status.FAILED) {
				printScanStatus(result, " scan(byte[]) ");

				throw new VirusException("Status: " + result.getStatus().name()
						+ " Result: " + result.getResult(),
						result.getException());
			}
			else if (result.getStatus() == ScanResult.Status.ERROR) {
				printScanStatus(result, " scan(byte[]) ");

				String strException = result.getException() != null ? result
						.getException().getMessage() : "";

				throw new InfraException(_MSG_ERROR_INFRA
						+ " problema processamento arquivo. " + strException);
			}
		} catch (IOException e) {
			_LOGGER.log(Level.WARNING,
					" ANTI-V\u00cdRUS N\u00c3O PODE ESCANEAR ARQUIVO!");
			e.printStackTrace();
		}
		return true;
	}

	/**
	 * @see br.unicamp.ccuec.ctmq.virusScan.spec.VirusScan#scan(java.io.InputStream)
	 */
	@Override
	public boolean scan(InputStream is) throws VirusException, InfraException {
		ScanResult result = clamav.scan(is);
		if (result.getStatus() == ScanResult.Status.FAILED) {
			printScanStatus(result, " scan(InputStream) ");

			throw new VirusException("Status: " + result.getStatus().name()
					+ " Result: " + result.getResult(), result.getException());
		}
		else if (result.getStatus() == ScanResult.Status.ERROR) {
			printScanStatus(result, " scan(InputStream) ");

			String strException = result.getException() != null ? result
					.getException().getMessage() : "";

			throw new InfraException(_MSG_ERROR_INFRA
					+ " problema processamento arquivo. " + strException);
		}
		return true;
	}
}